/*
 * File:        exiftool.c
 *
 * Description: Simple program to launch the pure Perl version of exiftool
 *
 * Usage:       exiftool [-h|normal exiftool arguments]
 *
 * Revisions:   2011/09/15 - P. Harvey Created
 *              2011/09/16 - Fixed mingw problem and use environment variables
 */

#define MINGW   /* comment out this line if not using the mingw compiler */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#ifdef MINGW
#include <process.h>
#endif
/*
 * define the default Perl executable and exiftool application script pathnames
 * (these may be changed at runtime by setting the corresponding environment variables)
 */
#define PERLNAME    "perl"
#define EXIFTOOL    "c:\\windows\\exiftool.pl"

int main(int argc, char **argv)
{
    int rtnVal = 0;

    /* generate our argument list */
    char **args = (char **)malloc((argc+2) * sizeof(char**));

    /* argument 0: perl executable name */
    args[0] = getenv("PERLNAME");
    if (!args[0]) args[0] = PERLNAME;

    /* argument 1: exiftool script name */
    args[1] = getenv("EXIFTOOL");
    if (!args[1]) args[1] = EXIFTOOL;

    if (argc == 2 && !strcmp(argv[1],"-h")) {
        printf("Name:   exiftool.exe - Run the Perl version of exiftool\n\n");
        printf("Environment variables:\n");
        printf("  PERLNAME - perl executable (\"%s\")\n", args[0]);
        printf("  EXIFTOOL - exiftool script (\"%s\")\n", args[1]);
        exit(0);
    }

    /* copy over the exiftool arguments */
    int i;
    for (i=1; i<argc; ++i) {
        args[i+1] = argv[i];
    }
    args[argc+1] = NULL;    /* NULL to end argument list */

#ifdef MINGW
    /* must use _spawnvp() instead of execvp() with mingw because with
     * mingw execvp() terminates the current process and forks a new one */
    rtnVal = _spawnvp(_P_WAIT, args[0], (const char* const*)args);
#else
    execvp(args[0], args);  /* should never return */
    rtnVal = 1;
#endif

    if (rtnVal) fprintf(stderr, "Error executing %s\n", args[0]);

    return(rtnVal);
}
